# For now, the User model is being persisted in two repositories:
#   * On postgres, using ActiveRecord
#   * OrientDb as graph nodes
#
# Goals: Keep the users in both (postgres, orientdb)
#
#               Devise Controllers -----> User model (with active record api)
#              /
#     Client -/
#             \
#              \
#               Portal Mec Controllers -> User repository
#
# How works?
#   For keep simple, user node in orientdb stored only a reference to the real user in postgres.
#   So, the user node will be only created or destroyed.
#
#    When devise create user, the active record hooks will sync with user repository
#    And, when the application call the user repository, it will keep sync with active record user model
class User < ActiveRecord::Base
  include RepositoriesProxy

  # Include default devise modules. Others available are:
  # :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable
  has_and_belongs_to_many :roles

  after_create SyncUserRepositoryService.new
  after_create :default_role

  after_destroy SyncUserRepositoryService.new
  after_destroy CreateUserBookmarksService.new

  has_attached_file :avatar, styles: {medium: "300x300>", thumb: "60x60>"}, default_url: lambda { |image| ActionController::Base.helpers.asset_path('user-anon.png') }
  validates_attachment_content_type :avatar, content_type: /\Aimage\/.*\Z/

  def collections
    public_collections = collection_repository.all(Collections::UserPublicContext.new(self))
    private_collections = collection_repository.all(Collections::UserPrivateContext.new(self))
    @collections = public_collections + private_collections
  end

  def institutions
    institution_repository.associated_with self
  end

  def bookmarks
    collection_repository.bookmarks self
  end

  def learning_objects
    learning_object_repository.all self
  end

  def is_admin?
    roles.each do |role|
      return true if role.name == "admin"
    end
    false
  end

  private

  def default_role
    roles << role_repository.find_by_name("teacher")
  end

end
